import * as i0 from '@angular/core';
import { Injectable, EventEmitter, Component, Input, Output, NgModule } from '@angular/core';
import { Subject, timer } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

class NgCircleTimerService {
    constructor() { }
}
NgCircleTimerService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgCircleTimerService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
NgCircleTimerService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgCircleTimerService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgCircleTimerService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return []; } });

class NgCircleTimerComponent {
    constructor() {
        this.duration = 0; // milliseconds
        this.color = '#1cbbf8';
        this.onComplete = new EventEmitter();
        this.destroy$ = new Subject();
        this.timeLeft = 0;
        this.ticking = false;
        this.completed = false;
        this.formattedTimeLeft = '';
        this.fullDasharray = 283;
        this.dasharray = `${this.fullDasharray} ${this.fullDasharray}`;
    }
    ngOnInit() {
        this.init(this.startDate);
    }
    ngOnDestroy() {
        this.destroy$.next();
        this.destroy$.complete();
    }
    init(startDate) {
        this.setTimeLeft(startDate);
        this.formatTimeLeft();
        this.setDasharray();
        if (this.timeLeft === 0) {
            this.completed = true;
        }
    }
    setTimeLeft(startDate) {
        this.ticking = false;
        this.completed = false;
        this.destroy$.next();
        if (!startDate) {
            this.timeLeft = this.duration;
            return 0;
        }
        const startTime = new Date(startDate).getTime();
        const endTime = startTime + this.duration;
        const timeLeftRaw = endTime - Date.now();
        if (timeLeftRaw <= 0) {
            this.timeLeft = 0;
            return 0;
        }
        const timeLeftSeconds = timeLeftRaw / 1000;
        this.timeLeft = Math.floor(timeLeftSeconds) * 1000;
        return timeLeftSeconds % 1;
    }
    start(startDate, delayMs = 0, replaying = false) {
        if (this.ticking) {
            console.log('Cannot start: timer already running.');
            return;
        }
        const decimalPortion = this.setTimeLeft(startDate);
        const startDelayMs = delayMs + decimalPortion * 1000;
        this.formatTimeLeft();
        this.setDasharray();
        if (this.timeLeft === 0) {
            this.completed = true;
            console.log('Cannot start: timer already completed.');
            return;
        }
        timer(startDelayMs, 1000)
            .pipe(takeUntil(this.destroy$))
            .subscribe(() => {
            this.ticking = true;
            this.timeLeft -= 1000;
            if (this.timeLeft <= 0) {
                this.timeLeft = 0;
                this.ticking = false;
                this.completed = true;
                this.onComplete.emit(replaying);
                this.destroy$.next();
            }
            this.formatTimeLeft();
            this.setDasharray();
        });
    }
    replay(startDate) {
        this.start(startDate, 1200, true);
    }
    pause() { }
    continue() { }
    complete() {
        this.timeLeft = 0;
        this.ticking = false;
        this.completed = true;
        this.destroy$.next();
        this.formatTimeLeft();
        this.setDasharray();
    }
    isTicking() {
        return this.ticking;
    }
    isCompleted() {
        return this.completed;
    }
    formatTimeLeft() {
        if (this.timeLeft <= 0) {
            this.timeLeft = 0;
        }
        const daysLeft = Math.floor(this.timeLeft / (1000 * 60 * 60 * 24));
        const hoursLeft = Math.floor((this.timeLeft % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60));
        const minutesLeft = Math.floor((this.timeLeft % (1000 * 60 * 60)) / (1000 * 60));
        const secondsLeft = Math.floor((this.timeLeft % (1000 * 60)) / 1000);
        const formattedDays = daysLeft < 10 ? `0${daysLeft}` : `${daysLeft}`;
        const formattedHours = hoursLeft < 10 ? `0${hoursLeft}` : `${hoursLeft}`;
        const formattedMinutes = minutesLeft < 10 ? `0${minutesLeft}` : `${minutesLeft}`;
        const formattedSeconds = secondsLeft < 10 ? `0${secondsLeft}` : `${secondsLeft}`;
        this.formattedTimeLeft = `${formattedMinutes}:${formattedSeconds}`;
        if (formattedHours !== '00' || formattedDays !== '00') {
            this.formattedTimeLeft = `${formattedHours}:` + this.formattedTimeLeft;
        }
        if (formattedDays !== '00') {
            this.formattedTimeLeft = `${formattedDays}:` + this.formattedTimeLeft;
        }
    }
    setDasharray() {
        const rawFraction = this.timeLeft / this.duration;
        const fraction = rawFraction - (1 / this.duration) * (1 - rawFraction);
        const remaining = Math.round(fraction * this.fullDasharray);
        this.dasharray = `${remaining} ${this.fullDasharray}`;
    }
}
NgCircleTimerComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgCircleTimerComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
NgCircleTimerComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.2.17", type: NgCircleTimerComponent, selector: "ng-circle-timer", inputs: { startDate: "startDate", duration: "duration", color: "color" }, outputs: { onComplete: "onComplete" }, ngImport: i0, template: `
    <div class="base-timer">
      <svg class="base-timer-svg"
           viewBox="0 0 100 100"
           xmlns="http://www.w3.org/2000/svg">
        <g class="base-timer-circle">
          <circle [class.base-timer-completed]=completed [class.base-timer-stroke]=!completed
                  cx="50"
                  cy="50"
                  r="45" />
          <path [attr.stroke-dasharray]="dasharray"
                [style.stroke]="color"
                id="remaining-time-stroke"
                d=" M 50, 50
                m -45, 0
                a 45,45 0 1,0 90,0
                a 45,45 0 1,0 -90,0">
          </path>
        </g>
      </svg>
      <span class="time">
    <span>{{formattedTimeLeft}}</span>
  </span>
    </div>
  `, isInline: true, styles: [":host {\n      .base-timer {\n        position: relative;\n        height: 200px;\n        width: 200px;\n\n        &-circle {\n          fill: none;\n          stroke: none;\n        }\n\n        &-stroke {\n          stroke-width: 1px;\n          stroke: grey;\n        }\n\n        &-completed {\n        // stroke-width: 2px;\n        // stroke: #88b462;\n\n          stroke-width: 1px;\n          stroke: grey;\n        }\n\n        .time {\n          position: absolute;\n          width: 200px;\n          height: 200px;\n          top: 0;\n          display: flex;\n          align-items: center;\n          justify-content: center;\n          font-size: 26px;\n\n          img {\n            height: 80px;\n          }\n        }\n\n        #remaining-time-stroke {\n          stroke-width: 2px;\n          transform: rotate(90deg);\n          transform-origin: center;\n          transition: 1s linear all;\n          stroke: #1cbbf8;\n        }\n\n        .base-timer-svg {\n          transform: scaleX(-1);\n        }\n      }\n    }\n    "] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgCircleTimerComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'ng-circle-timer',
                    template: `
    <div class="base-timer">
      <svg class="base-timer-svg"
           viewBox="0 0 100 100"
           xmlns="http://www.w3.org/2000/svg">
        <g class="base-timer-circle">
          <circle [class.base-timer-completed]=completed [class.base-timer-stroke]=!completed
                  cx="50"
                  cy="50"
                  r="45" />
          <path [attr.stroke-dasharray]="dasharray"
                [style.stroke]="color"
                id="remaining-time-stroke"
                d=" M 50, 50
                m -45, 0
                a 45,45 0 1,0 90,0
                a 45,45 0 1,0 -90,0">
          </path>
        </g>
      </svg>
      <span class="time">
    <span>{{formattedTimeLeft}}</span>
  </span>
    </div>
  `,
                    styles: [
                        `:host {
      .base-timer {
        position: relative;
        height: 200px;
        width: 200px;

        &-circle {
          fill: none;
          stroke: none;
        }

        &-stroke {
          stroke-width: 1px;
          stroke: grey;
        }

        &-completed {
        // stroke-width: 2px;
        // stroke: #88b462;

          stroke-width: 1px;
          stroke: grey;
        }

        .time {
          position: absolute;
          width: 200px;
          height: 200px;
          top: 0;
          display: flex;
          align-items: center;
          justify-content: center;
          font-size: 26px;

          img {
            height: 80px;
          }
        }

        #remaining-time-stroke {
          stroke-width: 2px;
          transform: rotate(90deg);
          transform-origin: center;
          transition: 1s linear all;
          stroke: #1cbbf8;
        }

        .base-timer-svg {
          transform: scaleX(-1);
        }
      }
    }
    `
                    ]
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { startDate: [{
                type: Input
            }], duration: [{
                type: Input
            }], color: [{
                type: Input
            }], onComplete: [{
                type: Output
            }] } });

class NgCircleTimerModule {
}
NgCircleTimerModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgCircleTimerModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
NgCircleTimerModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgCircleTimerModule, declarations: [NgCircleTimerComponent], exports: [NgCircleTimerComponent] });
NgCircleTimerModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgCircleTimerModule, imports: [[]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgCircleTimerModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        NgCircleTimerComponent
                    ],
                    imports: [],
                    exports: [
                        NgCircleTimerComponent
                    ]
                }]
        }] });

/*
 * Public API Surface of ng-circle-timer
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgCircleTimerComponent, NgCircleTimerModule, NgCircleTimerService };
//# sourceMappingURL=ng-circle-timer.js.map
